﻿----------------------------------------
-- Epic Gem List v 0.2, Copyright 2010 Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	Added blue cataclysm gems with test on first load
-- 0.3  Added JC only gems, cataclysm green and WotLK blue gems
-- 0.4	fixed loading of missing gems on first start (delayed GetItemInfo)
-- 0.5	added french translation, thanks to Araldwenn
-- 1.0	most translations from GlobalStrings.lua, quality and level selection, source cleanup, minor fixes
-- 1.1	new design, more detailed selections
-- 1.2	included BC gems
-- 1.3	small bugfixes (2 itemids), added debug mode (/egl debug)
-- 1.4b	added caching DB and options per char in SavedVariables
-- 1.5	show MarketPrice from Auctioneer, if installed
-- 1.6	new custom frame and menu design
-- 1.7	show details in result list
-- 1.80	temporary fix for not loading cataclysm gems
-- 1.82	saves favorite gems per char, fixed JC gem stats
-- 1.90	added recommendations for each class / spec
-- 2.00	added LibVersionCheck, shows all cataclysm gems
-- 2.01	added minimap button (for non LDB users)
-- 2.02	toggle minimap button. show gem colors in list
-- 2.03	added missing cataclysm gems, some small fixes
-- 2.04	temporarily removed auction house prices due to lua errors
-- 2.05	hopefully fixed delay on login
-- 2.06	some more tweaks which should speed up loading
-- 2.10 added new meta gems for 4.0.6, some classes changed
-- 2.11 added Vivid Dream Emerald
-- 2.12 added Reckless Hessonite
-- 2.20 added guild crafters for gems
-- 2.21 added config window
-- 2.22 small fixes
-- 2.23 toc change and filter fixed
-- 2.24 design fixed and 2 missing gems added
-- 2.30 added epic gems 4.3.0, some ugly fixes
-- 2.31 changed toc for 5.0.5 (will add data later)
----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------
EGL_TOPVER = "2";
EGL_SUBVER = "31";

EGL_VER = EGL_TOPVER.."."..EGL_SUBVER;
EGL_VERSION = "ver "..EGL_VER.." by Ydy @ Shattrath"

EGL_SOCKETSELECT = _G["COLOR"];
EGL_STATSELECT = _G["STATS_LABEL"];
EGL_CLOSE = _G["CLOSE"];
EGL_JC = EGL_GemLocaleText["jc"];

EGL_SCAN_FRAME_TITLE = "Epic Gem List";
EGL_SCAN_FRAME_SUBTITLE = "Retrieving Gem Data from server ...";
EGL_SCAN_FRAME_ABORT = "cancel";
EGL_SCAN_FRAME_INITIALIZING = "Initializing scan ...";
EGL_QUALITY = _G["QUALITY"];
EGL_ITEMLEVEL = _G["LEVEL"];

EGL_SHOWFAVORITES = EGL_GemLocaleText["fav"];
EGL_HIDEFAVORITES = _G["ALL"];  
EGLBUILD = nil;
EGLDBVER = nil;
egl_rs = nil;
EGL_DB_LOADED = nil;
tscurrent = 0;
tsmax = 0;
EGL_GC = {};
GC_STATUS = EGL_GemLocaleText["gcnotscanned"];

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local waitTable = {};
local waitFrame = nil;
local EGL_Scanned_Gems;
local MY_ADDON_NAME = "EpicGemList";
local MY_ADDON_VERSION = EGL_TOPVER..EGL_SUBVER;

-- INITIALIZE UNSET VARS ------------------------------------------------

if(EGL_Options == nil) then  -- saving options per char
    EGL_Options = {};
end

if(EGLDB == nil) then -- saving gem data per account
    EGLDB = {};
end

if (EGL_FAV == nil) then -- saving favorites per char
	EGL_FAV = {};
end

if (EGL_SSL == nil) then -- sorted localized class and spec list
	EGL_SSL = {};
end

if ( EGL_Options["SelectedStat"]  == nil ) then
    EGL_Options["SelectedStat"] = 0;
end

if ( EGL_Options["SelectedSlot"] == nil ) then
    EGL_Options["SelectedSlot"] = 0;
end

if ( EGL_Options["SelectedQualiOp"] == nil ) then
    EGL_Options["SelectedQualiOp"] = 1;
end

if ( EGL_Options["SelectedQuali"] == nil ) then
    EGL_Options["SelectedQuali"] = 2;
end

if ( EGL_Options["SelectedLevelOp"] == nil ) then
    EGL_Options["SelectedLevelOp"] = 1;
end

if ( EGL_Options["SelectedLevel"] == nil ) then
    EGL_Options["SelectedLevel"] = 2;
end

if ( EGL_Options["SelectedSpec"] == nil ) then
    EGL_Options["SelectedSpec"] = 0;
end

if (EGL_Options["jc"] == nil ) then
	EGL_Options["jc"] = false;
	EGL_Options["jc_cata"] = false;
end

if (EGL_Options["jc"] and EGL_Options["cataclysm"]) then
	EGL_Options["jc_cata"] = true;
end
if (EGL_Options["jc"] == false) then
	EGL_Options["jc_cata"] = false
end

if ( EGL_Options["buy"]  == nil ) then
    EGL_Options["buy"] = false;
end

if ( EGL_Options["loot"]  == nil ) then
    EGL_Options["loot"] = false;
end

if ( EGL_Options["details"]  == nil ) then
    EGL_Options["details"] = false;
end

if ( EGL_Options["favorites"] == nil ) then
	EGL_Options["favorites"] = false;
end

if ( EGL_Options["recommended"] == nil ) then
	EGL_Options["recommended"] = false;
end

if ( EGL_Options["primary"] == nil ) then
	EGL_Options["primary"] = false;
end

if (EGL_Options["debug"] == nil) then
	EGL_Options["debug"] = false
end

if (EGL_Options["minimap"] == nil) then
	EGL_Options["minimap"] = true
end

if (EGL_Options["gc"] == nil) then
	EGL_Options["gc"] = true
end

if (EGL_Scanned_Gems == nil) then
	EGL_Scanned_Gems = 0;
end
if (egl_scan_state == nil) then
	egl_scan_state = 0;
end
if (EGL_GemCount == nil) then
	EGL_GemCount = 0;
end
if (egl_shown_gems == nil) then
	egl_shown_gems = 0;
end


-- MISC FUNCTIONS ------------------------------------------------

---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
	--UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end

---- Delayed script execution ----
function EGL_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

--- counting table rows with every index format
function egl_tcount(tab)
   local n = #tab
   if (n == 0) then
     for _ in pairs(tab) do
       n = n + 1
     end
   end
   return n
end

--- Market Price from Auctioneer, if installed
function EGL_MarketPrice(itemID)
	if not itemID then
		if EGL_Options["debug"] then
			return "noItemID";
		end;
		return "";
	end
	aucinstalled = false;
	itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(itemID);
	if AucAdvanced then -- Auctioneer
		sellPrice, numSeen = AucAdvanced.API.GetMarketValue(itemLink);
		aucinstalled = true;
	-- elseif Atr_GetAuctionBuyout then -- Auctionator
		-- sellPrice = Atr_GetAuctionBuyout(itemID);
		-- aucinstalled = true;
	-- elseif AuctionLite then -- AuctionLite
		-- local hist = AuctionLite:GetHistoricalPriceById(itemID, 0);
		-- sellPrice = math.floor(hist.price);
		-- aucinstalled = true;
	-- elseif IsAddOnLoaded("AuctionMaster") and (AucMasStatisticVersion and AucMasStatisticVersion.major==1) then -- AuctionMaster
		-- avgBid, sellPrice, numBid, numBuy = AucMasGetAuctionInfo(itemLink, false);
		-- aucinstalled = true;	
	end;
	if sellPrice == nil then
		return "";
	else
		sellPrice = tonumber(sellPrice);
	end
	if aucinstalled then
		if (sellPrice ~= nil) and (sellPrice > 0) then
			local g = math.floor(sellPrice/10000);
			local GSC_GOLD = "ff807000"
			local digits = math.floor(math.log10(sellPrice)+1)
			if digits < 9 then
				gsc = string.format("|c%s%5d|r", GSC_GOLD, g)
			else
				gsc = string.format("|c%s%2.1fk|r", GSC_GOLD, g/1000)
			end
			if g == 0 then
				return "";
			end
			return gsc;
		end
	end
	return "";
end

--- scan 2. line of tooltip ---
function EGL_GetToolTipStats(itemid)
	ScanTooltip:ClearLines();
	ScanTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
    ScanTooltip:SetHyperlink("item:"..itemid..":0:0:0");
	itemStats = "";
	for i=2,ScanTooltip:NumLines() do
		local mytext=_G["ScanTooltipTextLeft"..i];
		local text=mytext:GetText();
		if i == 2 then
			itemStats = text; -- use first line on default
		end
		if string.find(text, "+") then
			itemStats = text; -- use line with + in text for stats
		end
 	end
	return itemStats;
end

--- LibVersionCheck ---

EGLAddon = {}
local lvcLib = LibStub:GetLibrary( "LibVersionCheck-1.0" )
lvcLib:RegisterVersion( MY_ADDON_NAME, MY_ADDON_VERSION, EGLAddon, "EGLVersionCallback" )
local newestVersion = tonumber( strmatch( MY_ADDON_VERSION, "%d+" ) )

function EGLAddon:EGLVersionCallback( sender, identifier, version )
    if identifier == MY_ADDON_NAME and version then
        local thisVersion = tonumber( strmatch( version, "%d+" ) )
        if thisVersion > newestVersion then
            newestVersion = thisVersion
            DEFAULT_CHAT_FRAME:AddMessage( string.format( "%s has a newer version of %s. (%s)", sender, MY_ADDON_NAME, version ) )
			DEFAULT_CHAT_FRAME:AddMessage("Download from http://wow.curse.com/downloads/wow-addons/details/egl.aspx");
        end
    end
end

--- Guild Member Is Online ? ---


function EGL_GuildMemberOnline(name)
	for i = 1, GetNumGuildMembers() do
		local toonName, rank, rankindex, level, class, zoneName, note, onote, connected, status = GetGuildRosterInfo(i)
		if connected then
			if toonName == name then
				return true;
			end
		end
	end
	return false;
end


--- GemColors ---

function EGL_GemColor(value)
	local red = nil;
	local blue = nil;
	local yellow = nil;


	if EGL_Data[value] ~= nil then
		red = EGL_Data[value]["red"];
		blue = EGL_Data[value]["blue"];
		yellow = EGL_Data[value]["yellow"];
	elseif EGL_Data_JC[value] ~= nil then
		red = EGL_Data_JC[value]["red"];
		blue = EGL_Data_JC[value]["blue"];
		yellow = EGL_Data_JC[value]["yellow"];
	elseif EGL_Data_Cata[value] ~= nil then
		red = EGL_Data_Cata[value]["red"];
		blue = EGL_Data_Cata[value]["blue"];
		yellow = EGL_Data_Cata[value]["yellow"];
	elseif EGL_Data_Cata_JC[value] ~= nil then
		red = EGL_Data_Cata_JC[value]["red"];
		blue = EGL_Data_Cata_JC[value]["blue"];
		yellow = EGL_Data_Cata_JC[value]["yellow"];
	elseif EGL_Data_BC[value] ~= nil then
		red = EGL_Data_BC[value]["red"];
		blue = EGL_Data_BC[value]["blue"];
		yellow = EGL_Data_BC[value]["yellow"];
	end
	return red, blue, yellow;
end

-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Gem List", {
    type = "launcher",
	label = "EpicGemList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EGL_Form:IsVisible() then
		        EGL_Form:Hide()
		    else
		        EGL_Form:Show()
	        end
        elseif msg == "RightButton" then
			if not EGL_Config:IsVisible() then
				EGL_Config:Show()
			else
				EGL_Config:Hide()
			end
		end
    end,
	icon = "Interface\\AddOns\\EpicGemList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Gem List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EGL_SlashCommandHandler(msg)
	if (msg == "debug") then
		if EGL_Options["debug"] then
			EGL_Options["debug"] = false;
			out("EGL: Debugging disabled");
		else
			EGL_Options["debug"] = true;
			if (EGL_Form:IsVisible() == false) then
				EGL_Form:Show()
			end
			
			out("EGL: Debugging enabled");
		end
	elseif (msg == "cata") then
		if EGL_Options["cataclysm"] then
			EGL_Options["cataclysm"] = false;
			out("EGL: Cataclysm support disabled");
		else
			EGL_Options["cataclysm"] = true;
			out("EGL: Cataclysm support enabled");
		end
	elseif (msg == "minimap") then
		if EGL_MinimapButton:IsVisible() then
			EGL_Options["minimap"] = false;
			EGL_MinimapButton:Hide();
		else
			EGL_Options["minimap"] = true;
			EGL_MinimapButton:Show();
		end
	elseif (msg == "scan") then
		EGL_ForceScan();
		if (EGL_Form:IsVisible() == false) then
			EGL_Form:Show();
		end
	elseif (msg == "gc") then
		if EGL_Options["gc"] then
			EGL_Options["gc"] = false;
			out("EGL: NOT scanning for guild crafters");
		else
			EGL_Options["gc"] = true;
			out("EGL: scanning for guild crafters ENABLED");
		end
	elseif (msg == "gcscan") then
		EGL_TradeSkill_Update()
	elseif (msg == "show") then
		EGL_Form:Show()
	elseif (msg == "hide") then
		EGL_Form:Hide()
	elseif (msg == "config") then
		if not EGL_Config:IsVisible() then
			EGL_Config:Show()
		else
			EGL_Config:Hide()
		end
	else
		out("|cFFFFFF00Epic Gem List: ");
		out("|cFFFFFF00("..EGL_VERSION..")");
		out("|cFF00FF00/egl show |cFFFFFFFF[show main window]");
		out("|cFF00FF00/egl hide |cFFFFFFFF[hide main window]");
		out("|cFF00FF00/egl config |cFFFFFFFF[show config window]");
		out("|cFF00FF00/egl scan |cFFFFFFFF[rescan and rebuild gem database]");
		out("|cFF00FF00/egl minimap |cFFFFFFFF[show/hide minimap button]");
		out("|cFF00FF00/egl gc |cFFFFFFFF[toggle guild crafting scan]");
		out("|cFF00FF00/egl gcscan |cFFFFFFFF[force guild crafting scan]");
		out("|cFF00FF00/egl debug |cFFFFFFFF[show/hide debugging messages]");
	end
end

-- ON LOAD ------------------------------------------------

function EGL_OnLoad(this)
	EGL_Form:RegisterEvent("VARIABLES_LOADED");
	eglf = _G["EGL_Form"];
	SLASH_EGL1 = "/epicgemlist";
	SLASH_EGL2 = "/egl";
	SlashCmdList["EGL"] = function(msg)
		EGL_SlashCommandHandler(msg);
	end
	if EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
	EGL_Options = {};
	EGL_Options["SelectedSlot"] = 0;
	EGL_Options["SelectedStat"] = 0;
	EGL_Options["SelectedQualiOp"] = 1;
	EGL_Options["SelectedQuali"] = 2;
	EGL_Options["SelectedLevelOp"] = 1;
	EGL_Options["SelectedLevel"] = 2;
	EGL_Options["SelectedSpec"] = 0;
	EGL_Options["debug"] = false;
	EGL_Options["details"] = false;
	EGL_Options["favorites"] = false;
	EGL_Options["cataclysm"] = true;
	EGL_Options["gc"] = false;
	EGL_Scanned_Gems = 0;
	local version, build, builddate, tocversion = GetBuildInfo();
end

-- ON SHOW ------------------------------------------------

function EGL_OnShow()
	if EGL_Options["jc"] then
		CheckButtonJC:SetChecked(true);
	else
		CheckButtonJC:SetChecked(false);
	end
	if EGL_Options["details"] then
		CheckButtonDetails:SetChecked(true);
	else
		CheckButtonDetails:SetChecked(false);
	end	
	local favb = _G["EGL_FavButton"];
	if EGL_Options["favorites"] == true then
		favb:SetText(EGL_HIDEFAVORITES);
	else
		favb:SetText(EGL_SHOWFAVORITES);
	end	
	for i = 1, 15 do
		item = _G["EGLItem_"..i];
		item:Hide();
	end
	EGL_ScrollBar_Update();
end

-- ON EVENT ------------------------------------------------

function EGL_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		local version, build, builddate, tocversion = GetBuildInfo();
		local mybuild = version..build;
		EGL_DB_LOADED = true;
		totalgems = 0;
		egl_bc = egl_tcount(EGL_Data_BC);
		egl_wotlk = egl_tcount(EGL_Data);
		egl_wotlk_jc = egl_tcount(EGL_Data_JC);
		egl_cata = egl_tcount(EGL_Data_Cata);
		egl_cata_jc = egl_tcount(EGL_Data_Cata_JC);
		totalgems = egl_bc + egl_wotlk + egl_wotlk_jc + egl_cata + egl_cata_jc;
		out("EpicGemList loaded ("..totalgems.." gems) ...");
		EGL_Options["cataclysm"] = true;
		if (EGLBUILD == nil) or (EGLDBVER == nil) then
			EGLBUILD = mybuild;
			EGLDBVER = EGL_VER;
			out("EGL: Scanning gems for local database ...");
			EGL_GemScan();
		end
		if (EGLBUILD ~= mybuild) or (EGLDBVER < EGL_VER) then
			EGLBUILD = mybuild;
			EGLDBVER = EGL_VER;
			out("EGL: Scanning gems for new WOW-Client "..version.." (Build "..build.." / "..builddate..") and EGL "..EGL_VER);
			EGL_GemScan();
		end
		EGL_Options["cataclysm"] = true;
		if EGL_Options["jc"] then
			CheckButtonJC:SetChecked(true);
		else
			CheckButtonJC:SetChecked(false);
		end
		if EGL_Options["gc"] then
			CheckButtonConfig3:SetChecked(true);
		else
			CheckButtonConfig3:SetChecked(false);
		end
		if EGL_Options["minimap"] == true then
			EGL_MinimapButton:Show()
			CheckButtonConfig1:SetChecked(true);
		else
			EGL_MinimapButton:Hide()
			CheckButtonConfig1:SetChecked(false);
		end
		if EGL_Options["debug"] == true then
			CheckButtonConfig5:SetChecked(true);
		else
			CheckButtonConfig5:SetChecked(false);
		end
		EGL_SlotDropDown_Init();
		EGL_StatDropDown_Init();
		EGL_ScrollBar_Update();
-- Disable initial guild scan due to massive problems with interface		
--		if EGL_Options["gc"] == true then
--			EGL_TradeSkill_Update();
--		end
	elseif ( event == "GUILD_RECIPE_KNOWN_BY_MEMBERS") then
		ViewGuildRecipes(755)
		--TradeSkillGuildFrame:Show();
		local skillLineID, recipeID, numMembers = GetGuildRecipeInfoPostQuery();
		--out(" ID: "..recipeID.." Members: "..numMembers)
		--out("GUILD_RECIPE_KNOWN_BY_MEMBERS")
		EGL_NextSkill()
		--TradeSkillGuildFrame:Hide()
		HideUIPanel(TradeSkillFrame)
		if IsAddOnLoaded("GnomeWorks") then
			if GnomeWorks.MainWindow then
				GnomeWorks.MainWindow:Hide()
			end
		end
	end
end

-- GET GUILDCRAFTERS -------------------------------------------


function EGL_GC_Scan_OnEnter(this)
	local entries = table.getn(EGL_HELP_GC);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_HELP_GC[i]);
	end
	EGLTooltip:AddLine(" ")
	EGLTooltip:AddLine(GC_STATUS)
    EGLTooltip:Show();
end

function EGL_GC_Scan_OnLeave()
   EGLTooltip:Hide();
end


function EGL_NextSkill()
	if tscurrent and tsmax then
		if tscurrent <= tsmax then			
			skillName, skillType, numAvailable, isExpanded, serviceType, numSkillUps = GetTradeSkillInfo(tscurrent)
			while skillType == "header" do
				tscurrent = tscurrent + 1
				skillName, skillType, numAvailable, isExpanded, serviceType, numSkillUps = GetTradeSkillInfo(tscurrent)
			end
			ilink = GetTradeSkillItemLink(tscurrent);
			if ilink then
				local _,itemId = strsplit(":", ilink)
				if itemId then
					itemId = tonumber(itemId)
					if (EGL_Data_Cata[itemId] ~= nil) or (EGL_Data[itemId] ~= nil) or (EGL_Data_BC[itemId] ~=nil) or (EGL_Data_Cata_JC[itemId] ~= nil) or (EGL_Data_JC[itemId] ~= nil) then
						local skillLineID, recipeID, numMembers = GetGuildRecipeInfoPostQuery();
						local members = {}
						for k = 1, numMembers do
							ison = "|cffff0000OFFLINE"
							name, isOnline = GetGuildRecipeMember(k);
							if isOnline then
								ison = "|cff00ff00ONLINE"
							end
							members[k] = name
							--out(itemId.." |cffffffff"..name.."  "..ison)
						end
						EGL_GC[itemId] = members
						local gccount = 0
						gccount = egl_tcount(EGL_GC)
						if egl_tcount(EGL_GC) > 0 then
							GC_STATUS = gccount.." "..EGL_GemLocaleText["gcfound"]
						end
					end
				end
			end
			tscurrent = tscurrent + 1
			skillName, skillType, numAvailable, isExpanded, serviceType, numSkillUps = GetTradeSkillInfo(tscurrent)
			while skillType == "header" do
				tscurrent = tscurrent + 1
				skillName, skillType, numAvailable, isExpanded, serviceType, numSkillUps = GetTradeSkillInfo(tscurrent)
			end
			SelectTradeSkill(tscurrent);
			TradeSkillFrame.selectedSkill = tscurrent;
			TradeSkillGuildFrame.queriedSkill = tscurrecnt;
			QueryGuildMembersForRecipe()
		end
		if tscurrent == tsmax then
			out("EGL: "..tsmax.." recipes scanned for guild crafters ...")
			EGL_ScrollBar_Update();
			local gccount = 0
			gccount = egl_tcount(EGL_GC)
			if egl_tcount(EGL_GC) > 0 then
				GC_STATUS = gccount.." "..EGL_GemLocaleText["gcfound"]
			else
				GC_STATUS = EGL_GemLocaleText["gcnogems"];
			end
		end
	end
end


function EGL_TradeSkill_Update()
	out("EGL: Scanning guild crafters ...")
	QueryGuildRecipes();
	local numTradeSkill = GetNumGuildTradeSkill();
	if numTradeSkill > 0 then
		for i = 1, numTradeSkill do
			local skillID, isCollapsed, iconTexture, headerName, numOnline, numPlayers, playerName, class, online, zone, skill, classFileName = GetGuildTradeSkillInfo(i);	
			if ( headerName and numPlayers > 0 ) then
				if ( CanViewGuildRecipes(skillID) ) then
					if skillID == 755 then -- Juwe
						ViewGuildRecipes(755)
						--GetGuildMemberRecipes(playerName, skillID);
						tsmax = GetNumTradeSkills();
						--tsmax = 50; -- zum Testen
						tscurrent = GetFirstTradeSkill();
						SelectTradeSkill(tscurrent);
						TradeSkillFrame.selectedSkill = tscurrent;
						TradeSkillGuildFrame.queriedSkill = tscurrecnt;
						QueryGuildMembersForRecipe()
						HideUIPanel(TradeSkillFrame)
						GC_STATUS = EGL_GemLocaleText["gcscanning"];
						return;
					end
				else
					if skillID == 755 then -- Juwe
						GC_STATUS = EGL_GemLocaleText["gcmissing"];
					end
				end
			else
				GC_STATUS = EGL_GemLocaleText["gcmissing"];
			end
		end
	else
		GC_STATUS = EGL_GemLocaleText["gcmissing"];
	end
end



-- GET GEM INFO ------------------------------------------------

function EGL_GemInfo(value)
	if EGL_DB_LOADED then
		if EGLDB[value] ~= nil then 
			local iteminfo = EGLDB[value];
			if iteminfo[6] == nil then
				iteminfo[6] = EGL_GetToolTipStats(value);
				EGLDB[value] = { iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6] }
			end
			return iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6];
		else
			EGL_GetGemInfo(value);
			return false;
		end
	else
		if EGL_Options["debug"] then
			out("EGL DEBUG: EGLDB not yet loaded");
		end
		return false;
	end
end
-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EGL_Help_OnEnter(this)
	local entries = table.getn(EGL_Help);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Help[i]);
	end
    EGLTooltip:Show();
end

function EGL_Help_OnLeave()
  EGLTooltip:Hide();
end
-- FAV BUTTON ------------------------------------------------

function EGL_Fav_OnEnter(this)
	local entries = table.getn(EGL_Fav);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Fav[i]);
	end
    EGLTooltip:Show();
end

function EGL_Fav_OnLeave()
  EGLTooltip:Hide();
end

function EGL_Fav_OnClick(this)
	local favb = _G["EGL_FavButton"];
	local recb = _G["EGL_RecButton"];
	if EGL_Options["favorites"] == true then
		EGL_Options["favorites"] = false;
		favb:SetText(EGL_SHOWFAVORITES);
	else
		EGL_Options["favorites"] = true;
		favb:SetText(ALL);
	end
	EGL_ScrollBar_Update();
end


-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EGLItem_OnEnter(this)
local addmembers = "GuildCrafters: ";

	function EGL_AddMember2Tooltip(_index)
		if EGL_GuildMemberOnline(members[_index]) then
		--if UnitIsConnected(members[_index]) then
			addmembers = addmembers.."|cff00ff00"..members[_index]
		else
			addmembers = addmembers.."|cffff0000"..members[_index]
		end
		addmembers = addmembers.."|cffffffff, ";
	end

	EGLTooltip:ClearLines();
	local id = this:GetID()+ FauxScrollFrame_GetOffset(EGL_ScrollBar);  
	if id ~= nil then
		local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);
		EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
		EGLTooltip:SetHyperlink("item:"..EGL[id]..":0:0:0");
		EGLTooltip:AddLine("Item Level: "..itemLevel..", ID: "..EGL[id], 0.7,0.7,1);
		if EGL_GC[EGL[id]] then
			EGLTooltip:AddLine(" ");
			members = EGL_GC[EGL[id]]
			table.foreach(members, EGL_AddMember2Tooltip)
			EGLTooltip:AddLine(addmembers);
		end
		EGLTooltip:Show();
	end 
end

function EGLItem_OnLeave()
  SELECTEDITEM = 0;
  EGLTooltip:Hide();
end

function EGLItem_OnClick(this)
	local EGL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EGL_ScrollBar);
	local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);

	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and itemName) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
	if IsControlKeyDown() then
		if EGL_FAV[EGL[id]] ~= nil then
			EGL_FAV[EGL[id]] = nil;
			EGL_ScrollBar_Update();
		else
			EGL_FAV[EGL[id]] = true;
			EGL_ScrollBar_Update();
		end
	end
end


-- RESULT LIST ------------------------------------------------



function EGL_HasNotStat(egltable,value)
	if EGL_Options["SelectedStat"] ~= 0 and 
	 (egltable[value]["stat1"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat2"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat3"] ~= EGL_Options["SelectedStat"]) then
		return true;
	else
		return false;
	end

end

function EGL_IsNotSlot(egltable,value)
	if EGL_Options["SelectedSlot"] ~= 0 and 
	  ((EGL_Options["SelectedSlot"] == 1 and egltable[value]["red"] == false) or
	  (EGL_Options["SelectedSlot"] == 2 and egltable[value]["blue"] == false) or
	  (EGL_Options["SelectedSlot"] == 3 and egltable[value]["yellow"] == false) or
	  (EGL_Options["SelectedSlot"] == 4 and egltable[value]["meta"] == false)) then
		return true;
	else
		return false;
	end
end

function EGL_IsNotQuali(value)
	if EGL_Options["SelectedQuali"] ~= 0 then
		local itemName, itemLink, itemRarity, itemLevel, itemTexture = EGL_GemInfo(value);
		if itemRarity == nil then
			return false;
		end
		if EGL_Options["SelectedQualiOp"] == 0 then
			if (itemRarity ~= (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 1 then
			if (itemRarity < (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 2 then
			if (itemRarity > (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		end	
	end
	return false;
end

function EGL_ShowLevel(selected,required)
	if EGL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EGL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentgem;    
    EGL = {}; 
    currentgem = 1;
	egl_shown_gems = 0;
	if EGL_Options["SelectedSpec"] == nil then
		EGL_Options["SelectedSpec"] = 0;
	end
	
	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2) then -- ALL, WotLK
		for value,_ in pairs(EGL_Data) do    
			additem = true;
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			elseif EGL_Options["SelectedSpec"] > 0 then
				if EGL_Recommended(EGL_Data,value) == false then
					additem = false;
				end	
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
	
	if ((EGL_Options["cataclysm"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cataclysm
		for value,_ in pairs(EGL_Data_Cata) do    
			additem = true;	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			elseif EGL_Options["SelectedSpec"] > 0 then
				if EGL_Recommended(EGL_Data_Cata,value) == false then
					additem = false;
				end	
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end
	
	if ((EGL_Options["jc"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2))) then	-- jc + ALL, WotLK
		for value,_ in pairs(EGL_Data_JC) do    
			additem = true;	    	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			elseif EGL_Options["SelectedSpec"] > 0 then
				if EGL_Recommended(EGL_Data_JC,value) == false then
					additem = false;
				end	
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end

	if ((EGL_Options["jc_cata"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cata
		for value,_ in pairs(EGL_Data_Cata_JC) do    
			additem = true;	    	    
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			elseif EGL_Options["SelectedSpec"] > 0 then
				if EGL_Recommended(EGL_Data_Cata_JC,value) == false then
					additem = false;
				end
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end	

	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],1) then -- ALL, BC
		for value,_ in pairs(EGL_Data_BC) do    
			additem = true;	  
			if EGL_Options["favorites"] then
				if EGL_FAV[value] == nil then
					additem = false;
				end
			end
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_BC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_BC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			elseif EGL_Options["SelectedSpec"] > 0 then
				if EGL_Recommended(EGL_Data_BC,value) == false then
					additem = false;
				end	
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
    table.sort(EGL, EGL_SortResults);
    EGL_ShowGemList();    
end

function EGL_ScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc)
	if EGL_Options["details"] then
		lineheight = 32;
	end
	FauxScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc);
end


function EGL_SortResults(a, b)
	local iteminfoA = EGL_GemInfo(a);
	local iteminfoB = EGL_GemInfo(b);
    local itemNameA, itemLinkA, itemQualityA, itemLevelA = EGL_GemInfo(a);
    local itemNameB, itemLinkB, itemQualityB, itemLevelB = EGL_GemInfo(b);
	
	if ((iteminfoA ~= nil) and (iteminfoB ~= nil)) then
		if itemLevelA ~= itemLevelB then	-- Sort by item level first
			return itemLevelA > itemLevelB;
		end
		if itemQualityA ~= itemQualityB then	-- Sort by item quality second
			return itemQualityA > itemQualityB;
		end
		if itemNameA and itemNameB then		-- if same item level, sort by name
			return itemNameA < itemNameB;
		end
	end
	return a < b;						-- sort by id, if no name
end


function EGL_ShowGemList()  
	local i, item, itemm, itemn, itemi, itemf, itemr, itemb, itemy;
	local sg = 15;
    MaxItems = egl_tcount(EGL);
	if EGL_Options["details"] then
		lineheight = 32;
		sg = 7;
	else
		lineheight = 16;
		sg = 15;
	end
    FauxScrollFrame_Update(EGL_ScrollBar,MaxItems,sg,lineheight);
    for i=1,sg do
		-- get global button names from xml script
		item = _G["EGLItem_"..i];
		itemgc = _G["EGLItem_"..i.."_GC"];
		itemn = _G["EGLItem_"..i.."_Name"];
		itemi = _G["EGLItem_"..i.."_Icon"];
		itemf = _G["EGLItem_"..i.."_Fav"];
		itemp = _G["EGLItem_"..i.."_MarketPrice"];
		iteme = _G["EGLItem_"..i.."_Extra"];
		
		itemr = _G["EGLItem_"..i.."_Red"];
		itemb = _G["EGLItem_"..i.."_Blue"];
		itemy = _G["EGLItem_"..i.."_Yellow"];
		
        lineplusoffset = i + FauxScrollFrame_GetOffset(EGL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			-- get gem data from local DB or load from server if not available
			local itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats = EGL_GemInfo(EGL[lineplusoffset]);
			
			if itemb then
				if EGL_GC[EGL[lineplusoffset]] then
					members = EGL_GC[EGL[lineplusoffset]]
					local num = egl_tcount(members)
					local memon = "|cffff0000GC"
					for j=1,num do
						if EGL_GuildMemberOnline(members[j]) then
						--if UnitIsConnected(members[j]) then
							memon = "|cff00ff00GC"
						end
					end
					itemgc:SetText(memon);
				else
					itemgc:SetText("");
				end
			end
			
			if (itemStats == nil) or (itemStats == "") then
				itemStats = EGL_GetToolTipStats(EGL[lineplusoffset]);
			end
			if itemName then
				if iteme ~= nil then
					-- show detailed info in result list ?
					if EGL_Options["details"] then
						item:SetHeight(30);
						iteme:SetText(itemStats);
						iteme:Show();
					else
						item:SetHeight(15);
						iteme:SetText("");
						iteme:Hide();
					end
				end

				if itemn ~= nil then
					itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if itemp ~= nil then
					eglprice = EGL_MarketPrice(EGL[lineplusoffset]);
					if eglprice then
						itemp:SetText(eglprice);
					end
				end
				if itemf ~= nil then
					if EGL_FAV[EGL[lineplusoffset]] ~= nil then
						itemf:SetTexture("Interface/GROUPFRAME/UI-Group-MasterLooter");
						itemf:Show();
					else
						itemf:SetTexture("");
						itemf:Hide();
					end
				end
				if itemr ~= nil then
					local red, blue, yellow = EGL_GemColor(EGL[lineplusoffset]);
					if red then
						itemr:SetTexture("Interface\\AddOns\\EpicGemList\\Design\\Red");
						itemr:Show();
					else
						itemr:SetTexture("");
						itemr:Hide();
					end
					if blue then
						itemb:SetTexture("Interface\\AddOns\\EpicGemList\\Design\\Blue");
						itemb:Show();
					else
						itemb:SetTexture("");
						itemb:Hide();
					end
					if yellow then
						itemy:SetTexture("Interface\\AddOns\\EpicGemList\\Design\\Yellow");
						itemy:Show();
					else
						itemy:SetTexture("");
						itemy:Hide();
					end				
				end				
				if item ~= nil then
					item:Show();
				end

			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EGL_GEMCOUNT"];
	if itemm ~= nil then
		local EGL_AUSWAHL = "";
		if EGL_Options["favorites"] then
			EGL_AUSWAHL = " ("..EGL_SHOWFAVORITES..")";
		elseif EGL_Options["recommended"] then
			EGL_AUSWAHL = " ("..RECOMMENDED..")";
		elseif EGL_Options["primary"] then
			EGL_AUSWAHL = " ("..PRIMARY..")";
		end
		itemm:SetText(MaxItems.." gems"..EGL_AUSWAHL..":");
	end
end

-- INITIAL GEM SCAN ------------------------------------------------

function EGL_GetGemInfoDelayed(value)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		local itemStats = EGL_GetToolTipStats(value);
		EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };		
		EGL_ScrollBar_Update();
		if EGL_Options["debug"] then
			out("EGL DEBUG: Added delayed Item ID"..value);
		end
	else
		out("EGL DEBUG: Could not retrieve ItemID "..value.." from server, please report to addon author");
	end
end


function EGL_GetGemInfo(value)
	-- REQUEST FROM SERVER (should be ONLY this line)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		EGL_Scanned_Gems = EGL_Scanned_Gems + 1;
		--if EGLDB[value] == nil then -- save GemInfo to DB
			if EGL_Options["debug"] then
				out("EGL Debug: Scanning Tooltip for "..itemName);
			end
			local itemStats = EGL_GetToolTipStats(value);
			EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };
		--end
		return true;
	else
		EGL_wait(10,EGL_GetGemInfoDelayed,value);
		failed = failed + 1;
		if EGL_Options["debug"] then
			out("EGL DEBUG: Failed to get ItemID "..value);
		end
		return false;
	end
end

function EGL_GemScan()
	EGLDB = {};	
	EGL_Scanned_Gems = 0;
	out("EGL: Scanning gems ...")
	if EGL_Options["cataclysm"] then
		for value,_ in pairs(EGL_Data_Cata) do
			if EGL_Options["debug"] then
				out("EGL Debug: Scanning Cataclysm gems");
			end
			EGL_GetGemInfo(value);
		end
		for value,_ in pairs(EGL_Data_Cata_JC) do
			if EGL_Options["debug"] then
				out("EGL Debug: Scanning Cataclysm JC gems");
			end
			EGL_GetGemInfo(value);
		end				
	end
	for value,_ in pairs(EGL_Data) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning WotLK gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_JC) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning WotLK JC gems");
		end
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_BC) do
		if EGL_Options["debug"] then
			out("EGL Debug: Scanning BC gems");
		end
		EGL_GetGemInfo(value);
	end
	EGL_FinalizeScan();
end

function EGL_AbortScan()
	egl_scan_state=0;
end

function EGL_ForceScan()
	egl_scan_state = 0;
	EGL_GemScan();
end

function EGL_FinalizeScan()
	egl_scan_state=1;
	out("EGL: Scanned "..EGL_Scanned_Gems.." gems.");
	if (failed > 0) then
		out("EGL: Missing "..failed.." gems will be added in a few seconds ...");
		failed = 0;
	end
	EGL_ScrollBar_Update();
end


